@@ -1,6 +1,13 @@
 
 
-gdal_info <- function(filename, options="", print=TRUE, open_opt="", ...) {
+gdal_version <- function() {
+	#if (is.null(.terra_environment$options)) .init()
+	.gdalversion()
+}
+
+describe <- function(filename, options="", print=TRUE, open_opt="", ...) {
+	#if (is.null(.terra_environment$options)) .init()
+
 	options <- unique(trimws(options))
 	options <- options[options != ""]
 	if (length(options) > 0) {
@@ -23,7 +30,8 @@ gdal_info <- function(filename, options="", print=TRUE, open_opt="", ...) {
 }
 
 
-sds_info <- function(filename, ...) {
+describe_sds <- function(filename, ...) {
+	#if (is.null(.terra_environment$options)) .init()
 	x <- .sdinfo(filename)
 	if (length(x[[1]]) == 1 & length(x[[2]]) == 0) {
 		stop(x[[1]])
