@@ -12,30 +12,35 @@ function(x, index, fun, ..., filename="", overwrite=FALSE, wopt=list()) {
 	uin <- d[,2]
 	nms <- make.names(d[,1])
 
-	txtfun <- .makeTextFun(match.fun(fun))
+	txtfun <- .makeTextFun(fun)
 	if (inherits(txtfun, "character")) { 
 		if (txtfun %in% c("max", "min", "mean", "prod", "sum", "any", "all")) {
 			opt <- .runOptions(filename, overwrite, wopt)
-			na.rm <- isTRUE(list(...)$na.rm)
-			x@ptr <- x@ptr$apply(index, txtfun, na.rm, nms, opt)	
+			narm <- isTRUE(list(...)$na.rm)
+			x@ptr <- x@ptr$apply(index, txtfun, narm, nms, opt)	
 			return(show_messages(x, "tapp"))
 		}		
 	}
+	fun <- match.fun(fun)
 
 	nl <- nlyr(x)
 	ind <- rep_len(index, nl)
 	out <- rast(x)
 	nlyr(out) <- length(uin)
 	names(out) <- nms
+	readStart(x)
+	on.exit(readStop(x))
 	b <- writeStart(out, filename, overwrite, wopt)
 	for (i in 1:b$n) {
 		v <- readValues(x, b$row[i], b$nrows[i], 1, ncol(out), TRUE)
-		v <- lapply(uin, function(i, ...) apply(v[, ind==uin[i], drop=FALSE], 1, fun, ...))
+		# like this, na.rm is not passed to FUN
+		# v <- lapply(uin, function(j, ...) apply(v[, ind==uin[j], drop=FALSE], 1, FUN=fun, ...))
+		# like this it works
+		v <- lapply(uin, function(j) apply(v[, ind==uin[j], drop=FALSE], 1, FUN=fun, ...))
 		v <- do.call(cbind, v)
 		writeValues(out, v, b$row[i], b$nrows[i])
 	}
-	readStop(x)
-	out <- writeStop(out)
+	out <- writeStop(out)	
 	return(out)
 }
 )
