@@ -5,7 +5,8 @@ setMethod("zonal", signature(x="SpatRaster", z="SpatRaster"),
 		if (inherits(txtfun, "character")) { 
 			if (txtfun %in% c("max", "min", "mean", "sum")) {
 				na.rm <- isTRUE(list(...)$na.rm)
-				ptr <- x@ptr$zonal(z@ptr, txtfun, na.rm)
+				opt <- .getOptions()
+				ptr <- x@ptr$zonal(z@ptr, txtfun, na.rm, opt)
 				show_messages(ptr, "zonal")
 				return( .getSpatDF(ptr) )
 			}		
@@ -33,15 +34,28 @@ setMethod("zonal", signature(x="SpatRaster", z="SpatRaster"),
 
 
 setMethod("global", signature(x="SpatRaster"), 
-	function(x, fun="mean", ...)  {
+	function(x, fun="mean", weights=NULL, ...)  {
 
 		nms <- names(x)
 		nms <- make.unique(nms)
+		txtfun <- .makeTextFun(match.fun(fun))
+		opt <- .getOptions()
+		if (!is.null(weights)) {
+			stopifnot(inherits(weights, "SpatRaster"))
+			stopifnot(txtfun %in% c("mean", "sum"))
+			na.rm <- isTRUE(list(...)$na.rm)
+			ptr <- x@ptr$global_weighted_mean(weights@ptr, txtfun, na.rm, opt)
+			show_messages(ptr, "global")
+			res <- (.getSpatDF(ptr))
+			rownames(res) <- nms
+			return(res)
+		}
+		
 		txtfun <- .makeTextFun(match.fun(fun))
 		if (inherits(txtfun, "character")) { 
 			if (txtfun %in% c("max", "min", "mean", "sum")) {
 				na.rm <- isTRUE(list(...)$na.rm)
-				ptr <- x@ptr$global(txtfun, na.rm)
+				ptr <- x@ptr$global(txtfun, na.rm, opt)
 				show_messages(ptr, "global")
 				res <- (.getSpatDF(ptr))
 
