@@ -322,7 +322,7 @@ SpatRaster SpatRaster::subset(std::vector<unsigned> lyrs, SpatOptions &opt) {
             offset = 0;
             for (size_t i=0; i<ss; i++) { offset += lyrbys[i]; }
             slyr = { lyrs[i] - offset } ;
-       }
+		}
     }
 
     rs = source[ss].subset(slyr);
@@ -343,9 +343,13 @@ SpatRaster SpatRaster::subset(std::vector<unsigned> lyrs, SpatOptions &opt) {
 
 bool RasterSource::combine_sources(const RasterSource &x) {
 	if (memory & x.memory) {
-		values.insert(values.end(), x.values.begin(), x.values.end());
-		layers.resize(nlyr + x.nlyr);
-		std::iota(layers.begin(), layers.end(), 0);
+		if ((values.size() + x.values.size()) < (values.max_size()/8) ) {
+			values.insert(values.end(), x.values.begin(), x.values.end());
+			layers.resize(nlyr + x.nlyr);
+			std::iota(layers.begin(), layers.end(), 0);
+		} else {
+			return false;
+		}
 	} else if (filename == x.filename) {
 		layers.insert(layers.end(), x.layers.begin(), x.layers.end());
 	} else {
